package com.ruoyi.common.sms;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.dysmsapi.model.v20170525.SendSmsRequest;
import com.aliyuncs.dysmsapi.model.v20170525.SendSmsResponse;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.profile.DefaultProfile;
import com.aliyuncs.profile.IClientProfile;
import com.ruoyi.common.exception.ServiceException;

@Service
public class SmsAliService {
    // 产品名称:云通信短信API产品,开发者无需替换
    @Value("${sms.ali.product}")
    private String product;

    // 产品域名,开发者无需替换
    @Value("${sms.ali.domain}")
    private String domain;

    @Value("${sms.ali.accessKeyId}")
    private String accessKeyId;

    @Value("${sms.ali.accessKeySecret}")
    private String accessKeySecret;

    // 签名
    @Value("${sms.ali.signName}")
    private String signName;

    /**
     * 发送短信验证码
     */
    public SendSmsResponse send(SmsSendDTO dto) {
        try {
            // 可自助调整超时时间
            System.setProperty("sun.net.client.defaultConnectTimeout", "10000");
            System.setProperty("sun.net.client.defaultReadTimeout", "10000");
            // 初始化acsClient,暂不支持region化
            IClientProfile profile = DefaultProfile.getProfile("cn-hangzhou", accessKeyId, accessKeySecret);
            DefaultProfile.addEndpoint("cn-hangzhou", "cn-hangzhou", product, domain);
            IAcsClient acsClient = new DefaultAcsClient(profile);
            // 组装请求对象-具体描述见控制台-文档部分内容
            SendSmsRequest request = new SendSmsRequest();
            // 必填:待发送手机号
            request.setPhoneNumbers(dto.getMobile());
            // 必填:短信签名-可在短信控制台中找到
            request.setSignName(signName);
            // 必填:短信模板-可在短信控制台中找到
            request.setTemplateCode(dto.getTemplateCode());
            // 可选:模板中的变量替换JSON串,如模板内容为"亲爱的${name},您的验证码为${code}"时,此处的值为
            request.setTemplateParam(dto.getParam().toJSONString());
            // hint 此处可能会抛出异常，注意catch
            return acsClient.getAcsResponse(request);
        } catch (ClientException e) {
            throw new ServiceException("短信发送失败，错误信息：" + e.getMessage());
        }
    }

}
