package com.ruoyi.common.utils;

import java.math.BigDecimal;

/**
 * @author wjh
 * 2024/4/3
 */
public class NumberUtils {

    /**
     * 获取不为空的数字
     * @param value 值
     * @return 若为空，返回0。否则返回值
     */
    public static BigDecimal nonNullDecimal(String value) {
        return nonNullDecimal(value, BigDecimal.ZERO);
    }

    /**
     * 获取不为空的数字
     * @param value 值
     * @param defaultValue 默认值
     * @return 若为空，返回默认值。否则返回值
     */
    public static BigDecimal nonNullDecimal(String value, BigDecimal defaultValue) {
        if (StringUtils.isBlank(value)) {
            return defaultValue;
        }
        try {
            return new BigDecimal(value);
        } catch (Exception e) {
            return defaultValue;
        }
    }

    /**
     * 获取不为空的数字
     * @param value 值
     * @param defaultValue 默认值
     * @return 若为空，返回默认值。否则返回值
     */
    public static int nonNullInt(String value, int defaultValue) {
        if (StringUtils.isBlank(value)) {
            return defaultValue;
        }
        try {
            return Integer.parseInt(value);
        } catch (Exception e) {
            return defaultValue;
        }
    }

    public static int nonNullInt(String value) {
        return nonNullInt(value, 0);
    }

}
