package com.ruoyi.common.valid.face.enums;

import lombok.AllArgsConstructor;
import lombok.Getter;

/**
 * @author wjh
 * 2024/9/23
 */
@Getter
@AllArgsConstructor
public enum FaceIncorrect {
    /**
     *  100 比对成功，还要根据score的值来判断是否同一人
     *     101 身份证号码姓名不一致
     *     102 库中无此号
     *     103 身份核验成功，数据非法，公安已经进行了身份核验（姓名身份证号码一致，公安返回的错误: 除库中无照片（109），特征提取失败（110），检测到多于一张人脸(111)，人脸识别系统异常(106)，图片不合法(112)以外的情况
     *     104 数据非法，公安未进行身份核验前图片校验，公安返回的错误: 除照片质量不合格(107)，上传图片文件过大(108)，人像比对服务异常(113)以外的情况
     *     106 身份核验成功，人脸识别系统异常
     *     107 照片质量不合格
     *     108 上传图片文件过大
     *     109 身份核验成功，库中无照片
     *     110 身份核验成功，特征提取失败
     *     111 身份核验成功，检测到多于一张人脸
     *     112 身份核验成功，图片不合法
     *     113 人像比对服务异常
     *     103 身份核验成功，数据非法
     */
    SUCCESS(100, "比对成功"),
    ID_NAME_NOT_CONSISTENT(101, "身份证号码姓名不一致"),
    NO_THIS_ID(102, "库中无此号"),
    ILLEGAL_DATA(103, "数据非法，公安已经进行了身份核验"),
    ILLEGAL_DATA_BEFORE_VERIFY(104, "数据非法，公安未进行身份核验前图片校验"),
    FACE_RECOGNITION_SYSTEM_ERROR(106, "身份核验成功，人脸识别系统异常"),
    PHOTO_QUALITY_NOT_GOOD(107, "照片质量不合格"),
    UPLOAD_FILE_TOO_LARGE(108, "上传图片文件过大"),
    NO_PHOTO(109, "身份核验成功，库中无照片"),
    FEATURE_EXTRACTION_FAILED(110, "身份核验成功，特征提取失败"),
    MORE_THAN_ONE_FACE(111, "身份核验成功，检测到多于一张人脸"),
    ILLEGAL_PICTURE(112, "身份核验成功，图片不合法"),
    FACE_RECOGNITION_SERVICE_EXCEPTION(113, "人像比对服务异常");

    private final Integer code;
    private final String msg;

    public static final boolean isSuccess(Integer code ) {
        return SUCCESS.getCode().equals(code);
    }
}
