package com.ruoyi.bst.account.mapper;

import com.ruoyi.bst.account.domain.Account;
import com.ruoyi.bst.account.domain.AccountQuery;
import com.ruoyi.bst.account.domain.AccountVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 收款账户Mapper接口
 *
 * @author ruoyi
 * @date 2025-04-07
 */
public interface AccountMapper
{
    /**
     * 查询收款账户
     *
     * @param id 收款账户主键
     * @return 收款账户
     */
    AccountVO selectAccountById(Long id);

    /**
     * 查询收款账户列表
     *
     * @param query 收款账户
     * @return 收款账户集合
     */
    List<AccountVO> selectAccountList(@Param("query")AccountQuery query);

    /**
     * 新增收款账户
     *
     * @param account 收款账户
     * @return 结果
     */
    int insertAccount(Account account);

    /**
     * 修改收款账户
     *
     * @param account 收款账户
     * @return 结果
     */
    public int updateAccount(@Param("data") Account account);

    /**
     * 删除收款账户
     *
     * @param id 收款账户主键
     * @return 结果
     */
    int deleteAccountById(Long id);

    /**
     * 批量删除收款账户
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteAccountByIds(Long[] ids);

    /**
     * 逻辑删除收款账户
     *
     * @param ids 需要删除的收款账户主键集合
     * @return 结果
     */
    int logicDel(@Param("ids") List<Long> ids);

    /**
     * 查询ID列表
     * @param query
     * @return
     */
    List<Long> selectIdList(@Param("query") AccountQuery query);

    /**
     * 查询总数
     */
    int selectCount(@Param("query") AccountQuery query);
}
