package com.ruoyi.bst.article.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.bst.article.mapper.ArticleMapper;
import com.ruoyi.bst.article.domain.Article;
import com.ruoyi.bst.article.domain.ArticleVO;
import com.ruoyi.bst.article.domain.ArticleQuery;
import com.ruoyi.bst.article.service.ArticleService;

/**
 * 文章列表Service业务层处理
 *
 * @author ruoyi
 * @date 2025-05-14
 */
@Service
public class ArticleServiceImpl implements ArticleService
{
    @Autowired
    private ArticleMapper articleMapper;

    /**
     * 查询文章列表
     *
     * @param id 文章列表主键
     * @return 文章列表
     */
    @Override
    public ArticleVO selectArticleById(Long id)
    {
        return articleMapper.selectArticleById(id);
    }

    /**
     * 查询文章列表列表
     *
     * @param article 文章列表
     * @return 文章列表
     */
    @Override
    public List<ArticleVO> selectArticleList(ArticleQuery article)
    {
        return articleMapper.selectArticleList(article);
    }

    /**
     * 新增文章列表
     *
     * @param article 文章列表
     * @return 结果
     */
    @Override
    public int insertArticle(Article article)
    {
        article.setCreateTime(DateUtils.getNowDate());
        return articleMapper.insertArticle(article);
    }

    /**
     * 修改文章列表
     *
     * @param article 文章列表
     * @return 结果
     */
    @Override
    public int updateArticle(Article article)
    {
        article.setUpdateTime(DateUtils.getNowDate());
        return articleMapper.updateArticle(article);
    }

    @Override
    public List<ArticleVO> selectArticleByCategoryId(Long categoryId) {
        return articleMapper.selectChildrenById(categoryId);
    }

    /**
     * 批量删除文章列表
     *
     * @param ids 需要删除的文章列表主键
     * @return 结果
     */
    @Override
    public int deleteArticleByIds(Long[] ids)
    {
        return articleMapper.deleteArticleByIds(ids);
    }

    /**
     * 删除文章列表信息
     *
     * @param id 文章列表主键
     * @return 结果
     */
    @Override
    public int deleteArticleById(Long id)
    {
        return articleMapper.deleteArticleById(id);
    }
}
