package com.ruoyi.bst.bonus.mapper;

import com.ruoyi.bst.bonus.domain.Bonus;
import com.ruoyi.bst.bonus.domain.BonusQuery;
import com.ruoyi.bst.bonus.domain.BonusVO;
import com.ruoyi.bst.bonus.domain.vo.BonusDailyStatVO;
import com.ruoyi.common.domain.vo.LocalDateDecimalVO;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;

/**
 * 分成明细Mapper接口
 *
 * @author ruoyi
 * @date 2025-03-26
 */
public interface BonusMapper
{
    /**
     * 查询分成明细
     *
     * @param id 分成明细主键
     * @return 分成明细
     */
    BonusVO selectBonusById(Long id);

    /**
     * 查询分成明细列表
     *
     * @param query 分成明细
     * @return 分成明细集合
     */
    List<BonusVO> selectBonusList(@Param("query")BonusQuery query);

    /**
     * 新增分成明细
     *
     * @param bonus 分成明细
     * @return 结果
     */
    int insertBonus(Bonus bonus);

    /**
     * 修改分成明细
     *
     * @param bonus 分成明细
     * @return 结果
     */
    public int updateBonus(@Param("data") Bonus bonus);

    /**
     * 删除分成明细
     *
     * @param id 分成明细主键
     * @return 结果
     */
    int deleteBonusById(Long id);

    /**
     * 批量删除分成明细
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteBonusByIds(Long[] ids);

    /**
     * 批量插入分成明细
     *
     * @param list 分成明细列表
     * @return 结果
     */
    int batchInsert(@Param("list") List<? extends Bonus> list);

    /**
     * 预分成
     * @param list 分成明细列表
     * @return 结果
     */
    int prepay(@Param("list") List<? extends Bonus> list);

    /**
     * 分成打款
     * @param id 分成明细ID
     * @param amount 打款金额
     * @param payTime 打款时间
     * @return 结果
     */
    int pay(@Param("id") Long id, @Param("amount") BigDecimal amount, @Param("payTime") LocalDateTime payTime);

    /**
     * 当未出账时退款
     */
    int refundWhenInvalid(@Param("id") Long id, @Param("amount") BigDecimal amount);

    /**
     * 当未分成时退款
     */
    int refundWhenWaitDivide(@Param("id") Long id, @Param("amount") BigDecimal amount);

    /**
     * 当已分成时退款
     */
    int refundWhenDividend(@Param("id") Long id, @Param("amount") BigDecimal amount);

    /**
     * 查询分成数量
     * @param query 查询条件
     * @return 结果
     */
    int selectCount(@Param("query") BonusQuery query);

    /**
     * 查询分成总金额
     * @param query 查询条件
     * @return 结果
     */
    BigDecimal selectSumOfAmount(@Param("query") BonusQuery query);

    /**
     * 查询分成总退款金额
     * @param query 查询条件
     * @return 结果
     */
    BigDecimal selectSumOfRefundAmount(@Param("query") BonusQuery query);

    /**
     * 查询分成每日金额
     * @param query
     * @return
     */
    List<LocalDateDecimalVO> selectDailySumOfAmount(@Param("query") BonusQuery query);

    /**
     * 查询分成每日统计
     * @param query 查询条件
     * @param keys 查询字段
     * @return 结果
     */
    List<BonusDailyStatVO> selectDailyStat(@Param("query") BonusQuery query, @Param("keys") List<String> keys);

    /**
     * 查询待分成金额
     * @param query
     * @return
     */
    BigDecimal selectSumOfWaitAmount(@Param("query") BonusQuery query);
}
