package com.ruoyi.bst.commandLog.service;

import com.ruoyi.bst.commandLog.domain.CommandLog;
import com.ruoyi.bst.commandLog.domain.CommandLogQuery;
import com.ruoyi.bst.commandLog.domain.CommandLogVO;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.iot.interfaces.IotDevice;

import java.time.LocalDate;
import java.util.List;

/**
 * 命令日志Service接口
 *
 * @author ruoyi
 * @date 2025-04-03
 */
public interface CommandLogService
{
    /**
     * 查询命令日志
     *
     * @param id 命令日志主键
     * @return 命令日志
     */
    public CommandLogVO selectCommandLogById(Long id);

    /**
     * 查询命令日志列表
     *
     * @param commandLog 命令日志
     * @return 命令日志集合
     */
    public List<CommandLogVO> selectCommandLogList(CommandLogQuery commandLog);

    /**
     * 新增命令日志
     *
     * @param commandLog 命令日志
     * @return 结果
     */
    public int insertCommandLog(CommandLog commandLog);

    /**
     * 修改命令日志
     *
     * @param commandLog 命令日志
     * @return 结果
     */
    public int updateCommandLog(CommandLog commandLog);

    /**
     * 批量删除命令日志
     *
     * @param ids 需要删除的命令日志主键集合
     * @return 结果
     */
    public int deleteCommandLogByIds(Long[] ids);

    /**
     * 删除命令日志信息
     *
     * @param id 命令日志主键
     * @return 结果
     */
    public int deleteCommandLogById(Long id);

    /**
     * 插入一条API日志
     *
     * @param mac       MAC
     * @param command   命令
     * @param result    结果
     * @param reason    原因
     * @param loginUser
     * @param iotCode
     */
    int addApiLog(String mac, String command, String result, String reason, LoginUser loginUser, Integer iotCode);

    /**
     * 插入一条蓝牙命令日志
     */
    int addBluetoothLog(CommandLog po, LoginUser loginUser);

    /**
     * 批量插入
     */
    int batchInsert(List<CommandLog> list);

    /**
     * 清除指定日期之前的日志
     */
    int clearBeforeDateLater(LocalDate date);
}
