package com.ruoyi.bst.device.service;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import com.ruoyi.bst.bindRecord.domain.enums.BindRecordType;
import com.ruoyi.bst.booth.domain.Booth;
import com.ruoyi.bst.booth.domain.BoothQuery;
import com.ruoyi.bst.booth.domain.BoothVO;
import com.ruoyi.bst.device.domain.Device;
import com.ruoyi.bst.device.domain.DeviceVO;
import com.ruoyi.bst.device.domain.DeviceQuery;
import com.ruoyi.bst.device.domain.vo.DeviceMacSnVO;
import com.ruoyi.bst.store.domain.StoreVO;
import com.ruoyi.common.domain.vo.LongIntegerVO;


/**
 * 设备Service接口
 *
 * @author ruoyi
 * @date 2025-04-23
 */
public interface DeviceService
{
    /**
     * 查询设备
     *
     * @param deviceId 设备主键
     * @return 设备
     */
    public DeviceVO selectDeviceByDeviceId(Long id, boolean scope);

    default DeviceVO selectDeviceByDeviceId(Long id) {
        return this.selectDeviceByDeviceId(id, false);
    }

    /**
     * 查询一个
     */
    public DeviceVO selectOne(DeviceQuery query);

    /**
     * 查询设备列表
     *
     * @param device 设备
     * @return 设备集合
     */
    public List<DeviceVO> selectDeviceList(DeviceQuery device);

    /**
     * 新增设备
     *
     * @param device 设备
     * @return 结果
     */
    public int insertDevice(Device device);

    /**
     * 修改设备
     *
     * @param device 设备
     * @return 结果
     */
    public int updateDevice(Device device);

    /**
     * 批量删除设备
     *
     * @param deviceIds 需要删除的设备主键集合
     * @return 结果
     */
    public int deleteDeviceByDeviceIds(Long[] deviceIds);

    /**
     * 删除设备信息
     *
     * @param deviceId 设备主键
     * @return 结果
     */
    public int deleteDeviceByDeviceId(Long deviceId);

    public int logicDel(List<Long> ids);

    /**
     * 根据设备ID绑定商户
     * @param deviceId
     * @param mchId
     * @return
     */
    String bindMchById(Long deviceId, Long mchId);

    /**
     * 根据sn绑定商户
     * @param deviceNo
     * @param mchId
     * @return
     */
    String bindMchBySn(String deviceNo, Long mchId);


    /**
     * 商户解绑设备
     *
     * @param deviceId 设备id
     */
    int unbind(Long deviceId);

    DeviceVO selectByDeviceNo(String value);

    /**
     * 根据mac修改在线状态
     */
    int updateOnlineStatusByMac(String mac, String status);




    /**
     * 根据设备id绑定卡座
     * @param deviceId
     * @param booth
     * @return
     */
    String bindBoothById(Long deviceId, BoothVO booth);

    /**
     * 根据设备SN绑定卡座
     * @param deviceNo
     * @param booth
     * @return
     */
    String bindBoothBySN(String deviceNo,BoothVO booth);

    /**
     * 根据设备ID解除绑定
     * @param deviceId
     * @return
     */
    public void unbindBooth(Long deviceId);


    /**
     * 设置数据库爆灯时长
     * @param deviceId
     * @param duration
     * @return
     */
    public int setTimeDb(Long deviceId, Long duration);

    /**
     * 查询店铺相关设备
     * @param query
     * @return
     */
    List<LongIntegerVO> selectCommonCountMap(DeviceQuery query);

    /**
     * 更改设备所属店铺
     * @param device
     * @return
     */
    int updateDeviceStore(Device device);

    /**
     * 设备解绑店铺
     * @param deviceId
     * @param store
     */
    void unbindStore(Long deviceId, StoreVO store);

    /**
     * 根据mac查询SN
     *
     * @param macList
     * @return
     */
    public List<DeviceMacSnVO> listSnByMac(List<String> macList);

}
