package com.ruoyi.bst.model.service.impl;

import java.util.Collections;
import java.util.List;

import com.github.pagehelper.PageHelper;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.bst.model.mapper.ModelMapper;
import com.ruoyi.bst.model.domain.Model;
import com.ruoyi.bst.model.domain.ModelVO;
import com.ruoyi.bst.model.domain.ModelQuery;
import com.ruoyi.bst.model.service.ModelService;

/**
 * 型号列表Service业务层处理
 *
 * @author ruoyi
 * @date 2025-04-23
 */
@Service
public class ModelServiceImpl implements ModelService
{
    @Autowired
    private ModelMapper modelMapper;

    /**
     * 查询型号列表
     *
     * @param modelId 型号列表主键
     * @return 型号列表
     */
    @Override
    public ModelVO selectModelByModelId(Long modelId)
    {
        return modelMapper.selectModelByModelId(modelId);
    }

    /**
     * 查询型号列表列表
     *
     * @param model 型号列表
     * @return 型号列表
     */
    @Override
    public List<ModelVO> selectModelList(ModelQuery model)
    {
        return modelMapper.selectModelList(model);
    }

    /**
     * 新增型号列表
     *
     * @param model 型号列表
     * @return 结果
     */
    @Override
    public int insertModel(Model model)
    {
        model.setCreateTime(DateUtils.getNowDate());
        return modelMapper.insertModel(model);
    }

    /**
     * 修改型号列表
     *
     * @param model 型号列表
     * @return 结果
     */
    @Override
    public int updateModel(Model model)
    {
        return modelMapper.updateModel(model);
    }

    /**
     * 批量删除型号列表
     *
     * @param modelIds 需要删除的型号列表主键
     * @return 结果
     */
    @Override
    public int deleteModelByModelIds(Long[] modelIds)
    {
        return modelMapper.deleteModelByModelIds(modelIds);
    }

    /**
     * 删除型号列表信息
     *
     * @param modelId 型号列表主键
     * @return 结果
     */
    @Override
    public int deleteModelByModelId(Long modelId)
    {
        return modelMapper.deleteModelByModelId(modelId);
    }

    @Override
    public int logicDel(List<Long> ids) {
        return modelMapper.logicDel(ids);
    }


    @Override
    public ModelVO selectOne(ModelQuery query) {
        PageHelper.startPage(1,1);
        List<ModelVO> list = this.selectModelList(query);
        if (CollectionUtils.isEmptyElement(list)) {
            return null;
        }
        return list.get(0);
    }

    @Override
    public List<ModelVO> selectModelListByIds(List<Long> ids) {
        if (CollectionUtils.isEmptyElement(ids)) {
            return Collections.emptyList();
        }
        ModelQuery query = new ModelQuery();
        query.setModelIds(ids);
        return modelMapper.selectModelList(query);
    }
}
