package com.ruoyi.bst.order.mapper;

import java.math.BigDecimal;
import java.util.List;
import com.ruoyi.bst.order.domain.Order;
import com.ruoyi.bst.order.domain.OrderVO;
import com.ruoyi.bst.order.domain.OrderQuery;
import com.ruoyi.bst.order.domain.query.OrderRefundQuery;
import com.ruoyi.bst.order.domain.vo.OrderDailyRefundStatVO;
import com.ruoyi.bst.order.domain.vo.OrderDailyStatVO;
import com.ruoyi.bst.order.domain.vo.OrderRankVO;
import com.ruoyi.bst.refund.domain.RefundQuery;
import com.ruoyi.common.domain.vo.*;
import com.ruoyi.system.user.domain.vo.UserConsumeVO;
import io.swagger.models.auth.In;
import org.apache.ibatis.annotations.Param;

/**
 * 订单列表Mapper接口
 *
 * @author ruoyi
 * @date 2025-04-26
 */
public interface OrderMapper
{
    /**
     * 查询订单列表
     *
     * @param id 订单列表主键
     * @return 订单列表
     */
    OrderVO selectOrderById(@Param("id") Long id);

    /**
     * 查询订单列表列表
     *
     * @param query 订单列表
     * @return 订单列表集合
     */
    List<OrderVO> selectOrderList(@Param("query")OrderQuery query);

    /**
     * 新增订单列表
     *
     * @param order 订单列表
     * @return 结果
     */
    int insertOrder(Order order);

    /**
     * 批量新增订单列表
     */
    int batchInsert(@Param("list") List<? extends Order> list);

    /**
     * 批量修改订单列表
     */
    int batchUpdate(@Param("list") List<? extends Order> list);

    /**
     * 修改订单列表
     *
     * @param order 订单列表
     * @return 结果
     */
    public int updateOrder(@Param("data") Order order);

    /**
     * 删除订单列表
     *
     * @param id 订单列表主键
     * @return 结果
     */
    int deleteOrderById(Long id);

    /**
     * 批量删除订单列表
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteOrderByIds(Long[] ids);

    /**
     * 根据查询条件查询订单ID列表
     * @param query 查询条件
     * @return 订单ID列表
     */
    List<Long> selectIdByQuery(@Param("query") OrderQuery query);


    /**
     * 根据查询条件更新订单
     * @param data 更新数据
     * @param query 查询条件
     * @return 结果
     */
    int updateByQuery(@Param("data") Order data, @Param("query") OrderQuery query);

    OrderVO selectOrderByBstId(@Param("bstId") Long bstId);

    int addRefundNum(@Param("id") Long id, @Param("number") Integer number);

    List<LocalDateTimeIntegerVO> selectOrderCount(@Param("query")OrderQuery orderQuery);

    List<LongIntegerVO> selectCountGroupByStoreId(@Param("query") OrderQuery query);

    List<UserConsumeVO> selectUserIdsByStoreId(@Param("storeId") Long storeId);

    List<LongDecimalVO> selectSumOfAmountGroupByUserId(@Param("query")OrderQuery query);

    List<LongLocalDateTimeVO> assembleLastConsumeTime(@Param("query") OrderQuery query);

    /**
     * 查询用户数量
     * @param query
     * @return
     */
    Integer selectUserCount(@Param("query") OrderQuery query);

    List<LongIntegerVO> selectUserCountGroupByStore(@Param("query") OrderQuery query);

    /**
     * 查询订单总金额
     *
     * @param query 查询条件
     * @return 结果
     */
    BigDecimal selectSumOfActualTotalAmount(@Param("query") OrderQuery query);

    BigDecimal selectSumOfActualManageFee(OrderQuery query);

    BigDecimal selectSumOfActualDispatchFee(OrderQuery query);

    BigDecimal selectSumOfPayAmount(OrderQuery query);

    Integer selectCount(@Param("query") OrderQuery orderQuery);

    List<StringIntegerVO> selectStatusCount(OrderQuery query);

    /**
     *查询订单退款金额
     * @param query
     * @return
     */
    BigDecimal selectSumOfRefundAmount(@Param("query") OrderRefundQuery query);

    List<OrderDailyStatVO> selectDailyStat(OrderQuery query, List<String> keys);

    List<OrderDailyRefundStatVO> selectDailyRefundStat(RefundQuery query, List<String> keys);

    List<OrderRankVO> selectRank(OrderQuery query);

    BigDecimal selectSumOfActualAmount(OrderQuery query);


}
