package com.ruoyi.bst.pay.mapper;

import com.ruoyi.bst.pay.domain.Pay;
import com.ruoyi.bst.pay.domain.PayQuery;
import com.ruoyi.bst.pay.domain.PayVO;
import com.ruoyi.common.domain.vo.LocalDateDecimalVO;
import com.ruoyi.common.domain.vo.LongDecimalVO;
import com.ruoyi.common.domain.vo.LongIntegerVO;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;

/**
 * 支付订单Mapper接口
 *
 * @author ruoyi
 * @date 2025-03-26
 */
public interface PayMapper
{
    /**
     * 查询支付订单
     *
     * @param id 支付订单主键
     * @return 支付订单
     */
    PayVO selectPayById(Long id);

    /**
     * 查询支付订单列表
     *
     * @param query 支付订单
     * @return 支付订单集合
     */
    List<PayVO> selectPayList(@Param("query")PayQuery query);

    /**
     * 新增支付订单
     *
     * @param pay 支付订单
     * @return 结果
     */
    int insertPay(Pay pay);

    /**
     * 批量新增支付订单
     */
    int batchInsert(@Param("list") List<? extends Pay> list);

    /**
     * 批量修改支付订单
     */
    int batchUpdate(@Param("list") List<? extends Pay> list);

    /**
     * 修改支付订单
     *
     * @param pay 支付订单
     * @return 结果
     */
    public int updatePay(@Param("data") Pay pay);

    /**
     * 删除支付订单
     *
     * @param id 支付订单主键
     * @return 结果
     */
    int deletePayById(Long id);

    /**
     * 批量删除支付订单
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deletePayByIds(Long[] ids);

    int updateByQuery(@Param("data") Pay data, @Param("query") PayQuery query);

    /**
     * 增加退款中金额
     * @param id 支付单ID
     * @param amount 退款金额
     * @return 结果
     */
    int addRefundingAmount(@Param("id") Long id, @Param("amount") BigDecimal amount);

    /**
     * 记录退款金额
     * @param id 支付单ID
     * @param amount 退款金额
     * @return 结果
     */
    int recordRefundAmount(@Param("id") Long id, @Param("amount") BigDecimal amount);

    /**
     * 查询支付单号列表
     * @param query
     * @return
     */
    List<String> selectNoList(@Param("query") PayQuery query);

    List<LocalDateDecimalVO> selectPayAmount(@Param("query") PayQuery payQuery);

    /**
     * 按条件查询店铺金额
     * @param query
     * @return
     */
    List<LongDecimalVO> selectSumOfAmountGroupByStoreId(@Param("query") PayQuery query);
}
