package com.ruoyi.bst.refund.mapper;

import com.ruoyi.bst.refund.domain.Refund;
import com.ruoyi.bst.refund.domain.RefundQuery;
import com.ruoyi.bst.refund.domain.RefundVO;
import com.ruoyi.common.domain.vo.LocalDateDecimalVO;
import com.ruoyi.common.domain.vo.LongDecimalVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 退款Mapper接口
 *
 * @author ruoyi
 * @date 2025-03-28
 */
public interface RefundMapper
{
    /**
     * 查询退款
     *
     * @param id 退款主键
     * @return 退款
     */
    RefundVO selectRefundById(Long id);

    /**
     * 查询退款列表
     *
     * @param query 退款
     * @return 退款集合
     */
    List<RefundVO> selectRefundList(@Param("query")RefundQuery query);

    /**
     * 新增退款
     *
     * @param refund 退款
     * @return 结果
     */
    int insertRefund(Refund refund);


    /**
     * 修改退款
     *
     * @param refund 退款
     * @return 结果
     */
    public int updateRefund(@Param("data") Refund refund);

    /**
     * 删除退款
     *
     * @param id 退款主键
     * @return 结果
     */
    int deleteRefundById(Long id);

    /**
     * 批量删除退款
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteRefundByIds(Long[] ids);

    /**
     * 根据条件更新退款
     * @param data 退款
     * @param query 条件
     * @return 结果
     */
    int updateByQuery(@Param("data") Refund data, @Param("query") RefundQuery query);

    /**
     * 查询店铺退款金额
     * @param refundQuery
     * @return
     */
    List<LocalDateDecimalVO> selectRefundAmount(@Param("query") RefundQuery refundQuery);

    /**
     * 根据id查询店铺退款金额
     * @param refundQuery
     * @return
     */
    List<LongDecimalVO> selectSumOfRefundAmountGroupByStoreId(@Param("query") RefundQuery refundQuery);
}
