package com.ruoyi.bst.store.mapper;

import java.util.List;
import com.ruoyi.bst.store.domain.Store;
import com.ruoyi.bst.store.domain.StoreVO;
import com.ruoyi.bst.store.domain.StoreQuery;
import org.apache.ibatis.annotations.Param;
import org.springframework.security.core.parameters.P;

/**
 * 店铺Mapper接口
 *
 * @author ruoyi
 * @date 2025-04-18
 */
public interface StoreMapper
{
    /**
     * 查询店铺
     *
     * @param storeId 店铺主键
     * @return 店铺
     */
    StoreVO selectStoreByStoreId(Long storeId);

    /**
     * 查询店铺列表
     *
     * @param query 店铺
     * @return 店铺集合
     */
    List<StoreVO> selectStoreList(@Param("query")StoreQuery query);

    /**
     * 新增店铺
     *
     * @param store 店铺
     * @return 结果
     */
    int insertStore(Store store);

    /**
     * 批量新增店铺
     */
    int batchInsert(@Param("list") List<? extends Store> list);

    /**
     * 批量修改店铺
     */
    int batchUpdate(@Param("list") List<? extends Store> list);

    /**
     * 修改店铺
     *
     * @param store 店铺
     * @return 结果
     */
    public int updateStore(@Param("data") Store store);

    /**
     * 删除店铺
     *
     * @param storeId 店铺主键
     * @return 结果
     */
    int deleteStoreByStoreId(Long storeId);

    /**
     * 批量删除店铺
     *
     * @param storeIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteStoreByStoreIds(Long[] storeIds);

    int logicDel(@Param("ids") List<Long> ids);

    /**
     * 查询店铺数量
     * @param query
     * @return
     */
    Integer selectCount(@Param("query") StoreQuery query);

    List<Long> selectIdByQuery(@Param("query") StoreQuery query);
}
