package com.ruoyi.system.user.mapper;

import java.math.BigDecimal;
import java.util.List;

import com.ruoyi.bst.order.domain.OrderQuery;
import com.ruoyi.system.user.domain.vo.UserConsumeVO;
import org.apache.ibatis.annotations.Param;

import com.ruoyi.common.core.domain.entity.User;
import com.ruoyi.common.core.domain.vo.UserVO;
import com.ruoyi.system.user.domain.UserQuery;
import com.ruoyi.system.user.domain.vo.UserNameVO;

/**
 * 用户表 数据层
 *
 * @author ruoyi
 */
public interface UserMapper
{

    UserVO selectSimpleById(Long userId);

    /**
     * 根据条件分页查询用户列表
     *
     * @param sysUser 用户信息
     * @return 用户信息集合信息
     */
    public List<UserVO> selectUserList(UserQuery sysUser);

    /**
     * 根据条件分页查询已配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<UserVO> selectAllocatedList(User user);

    /**
     * 根据条件分页查询未分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<UserVO> selectUnallocatedList(User user);

    /**
     * 通过用户名查询用户
     *
     * @param userName 用户名
     * @return 用户对象信息
     */
    public UserVO selectUserByUserName(String userName);

    /**
     * 通过用户ID查询用户
     *
     * @param userId 用户ID
     * @return 用户对象信息
     */
    public UserVO selectUserById(Long userId);

    /**
     * 新增用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public int insertUser(User user);

    /**
     * 修改用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public int updateUser(User user);

    /**
     * 修改用户头像
     *
     * @param userName 用户名
     * @param avatar 头像地址
     * @return 结果
     */
    public int updateUserAvatar(@Param("userName") String userName, @Param("avatar") String avatar);

    /**
     * 重置用户密码
     *
     * @param userName 用户名
     * @param password 密码
     * @return 结果
     */
    public int resetUserPwd(@Param("userName") String userName, @Param("password") String password);

    /**
     * 通过用户ID删除用户
     *
     * @param userId 用户ID
     * @return 结果
     */
    public int deleteUserById(Long userId);

    /**
     * 批量删除用户信息
     *
     * @param userIds 需要删除的用户ID
     * @return 结果
     */
    public int deleteUserByIds(Long[] userIds);

    /**
     * 校验用户名称是否唯一
     *
     * @param userName 用户名称
     * @return 结果
     */
    public User checkUserNameUnique(String userName);

    /**
     * 校验手机号码是否唯一
     *
     * @param phonenumber 手机号码
     * @return 结果
     */
    public User checkPhoneUnique(String phonenumber);

    /**
     * 校验email是否唯一
     *
     * @param email 用户邮箱
     * @return 结果
     */
    public User checkEmailUnique(String email);

    /**
     * 查询用户数量
     */
    int selectCount(UserQuery query);

    /**
     * 查询用户名称列表
     */
    List<UserNameVO> selectNameList(UserQuery query);

    /**
     * 查询用户ID列表
     */
    List<Long> selectUserIds(UserQuery query);


    /**
     * 查询用户余额
     */
    BigDecimal selectBalanceForUpdate(@Param("userId") Long userId);

    /**
     * 增加用户余额
     */
    public int addBalance(@Param("userId") Long userId, @Param("amount") BigDecimal amount);

    /**
     * 减少用户余额
     */
    public int subtractBalance(@Param("userId") Long userId, @Param("amount") BigDecimal amount, @Param("checkBalance") boolean checkBalance);

    /**
     * 查询用户ID列表
     */
    public List<Long> selectIdList(UserQuery query);

    /**
     * 查询用户余额
     */
//    public BigDecimal selectSumOfBalance(UserQuery query);

    /**
     * 查询店铺下消费过的用户基本信息
     * @param query
     * @return
     */
    List<UserConsumeVO> selectUserIdsByStoreId(@Param("query") OrderQuery query);
}
